/* $Id: gram-fbsd.y,v 1.4 2005/06/17 16:58:13 igor Exp $ */

/*
 * Copyright (c) 2004 Igor Boehm <igor@bytelabs.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF MIND, USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

%{

#include <sys/types.h>

#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "fail.h"
#include "macros.h"
#include "pbutil.h"
#include "pkgobject.h"
#include "y.tab.h"
#include "scanner.h"
#include "sort.h"
#include "gram.h"

/* PROTOTYPES */
static void    	nextentry(void);
int            	yyparse(void);
int            	yylex(void);

/* in here we store all the info per Package */
static size_t   lineno = 1;
PkgObject      *elem;

%}

/* our tokens which we have defined in scan.l */
%token SIMPLEWORD 
%token DELIM 
%token EOL 

%%

/*
 * GRAMMER for FreeBSD
 * SYNTAX:
 * 	  distribution-name|port-path|installation-prefix|comment| \
 *  	  description-file|maintainer|categories|build-deps|run-deps \
 *  	  www-site|extract-depends|patch-depends|fetch-depends \n
 */


entries		: /* EMPTY */
	 	| entries entry { nextentry(); lineno++; } EOL 
		| entries error { nextentry(); lineno++; } EOL
	 	;

entry		: dist_name port_path inst_prefix comment desc_file 
		  maintainer categories build_deps run_deps www_site
		  ext_deps patch_deps fetch_deps
		;

dist_name	: SIMPLEWORD DELIM	{
			g_object_set(elem, "distrib-name", (char *)$1, NULL);
		}
		;
port_path	: SIMPLEWORD DELIM   {
			g_object_set(elem, "port-path", (char *)$1, NULL);
  		}
		;
inst_prefix	: SIMPLEWORD DELIM   {
			g_object_set(elem, "install-prefix", (char *)$1, NULL);
  		}
		| DELIM /* empty */
		;
comment		: SIMPLEWORD DELIM   {
			g_object_set(elem, "comment", (char *)$1, NULL);
  		}
		;
desc_file	: SIMPLEWORD DELIM   {
			g_object_set(elem, "descr-file", (char *)$1, NULL);
  		}
		;
maintainer	: SIMPLEWORD DELIM   {
			g_object_set(elem, "maintainer", (char *)$1, NULL);
  		}
		;
categories	: SIMPLEWORD DELIM   {
			char *ptr;
			/*
			 * XXX: BE CAREFUL WITH MULTIPLE CATEGORIES for 
			 * right now we only take the first category a 
			 * package falls into -> but there may be more
			 * than one For right now we also only take major 
			 * categories, so if we find something like 
			 * x11/xfce we will only grab the x11 part.
			 */
			if (!IS_NULL((ptr = strchr((char *)$1, ' '))) ||
			    !IS_NULL((ptr = strchr((char *)$1, '\t'))))
				*ptr++ = '\0';
			if (!IS_NULL((ptr = strchr((char *)$1, '/'))))
				*ptr++ = '\0';
 		        g_object_set(elem, "category", (char *)$1, NULL);
		}	
		| DELIM /* empty */
		;
build_deps	: SIMPLEWORD DELIM   {
  			pb_replace((char *)$1, ' ', '\n');
			g_object_set(elem, "bdep", (char *)$1, NULL);
  		} 
		| DELIM /* empty */
		;
run_deps        : SIMPLEWORD DELIM   {
  			pb_replace((char *)$1, ' ', '\n');
			g_object_set(elem, "rdep", (char *)$1, NULL);
  		} 
		| DELIM /* empty */
		;
www_site	: SIMPLEWORD DELIM   {
			g_object_set(elem, "www", (char *)$1, NULL);
  		} 
		| DELIM /* empty */
		;
ext_deps	: SIMPLEWORD DELIM   {
  			/* DO NOTHING FOR NOW */
		}
		| DELIM /* empty */
		;

patch_deps	: SIMPLEWORD DELIM   {
  			/* DO NOTHING FOR NOW */
		}
		| DELIM /* empty */
		;

fetch_deps	: SIMPLEWORD	     {	
  			/* DO NOTHING FOR NOw */
		}
		| SIMPLEWORD DELIM   {
			yyerror("%d: Port %s has too many elments.\n", 
				lineno, pkgobject_get_distrib_name(elem));	   
			YYERROR;
		}
		| /* empty */
		;
%%

/**
 * parse_index():
 *
 * Function which opens the ports INDEX file and calls yyparse()
 * to begin the parsing process.
 */
Boolean
parse_index(char *filename)
{
	extern FILE    *yyin;

	if (IS_NULL((yyin = fopen(filename, "r")))) { 
		pb_warning("%s: %s", strerror(errno), filename);
		return FALSE;
	}	
	/* INITIALISE LIST */
	pkgobject_create_list(linecount(filename));
	elem = pkgobject_get_list_head();
	yyparse();
	fclose(yyin);
	/* SORT IT ASCENDING AND BASED ON CATEGORY */
	pb_pkglist_sort((pkgobject_get_list_size() - 1));
	return TRUE;
}

/**
 * nextentry():
 *
 * Add the old entry to the list and ALLOCATE more memory for a new entry. 
 */
static void
nextentry(void)
{
        Boolean	inst_stat; 

	inst_stat = pkgobject_check_install_status(elem); 
	g_object_set(elem, "install-status", inst_stat, NULL);
	if (inst_stat) pkgobject_increment_installed_packages();
	g_object_set(elem, "ID", lineno, NULL);
	if (pb_get_verbose_level() > 1) pkgobject_print(elem);
	elem = pkgobject_get_list_next(elem);
}

